'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');

function create$1(createState) {
  var state;
  var listeners = new Set();

  var setState = function setState(partial, replace) {
    var nextState = typeof partial === 'function' ? partial(state) : partial;

    if (nextState !== state) {
      var _previousState = state;
      state = replace ? nextState : Object.assign({}, state, nextState);
      listeners.forEach(function (listener) {
        return listener(state, _previousState);
      });
    }
  };

  var getState = function getState() {
    return state;
  };

  var subscribeWithSelector = function subscribeWithSelector(listener, selector, equalityFn) {
    if (selector === void 0) {
      selector = getState;
    }

    if (equalityFn === void 0) {
      equalityFn = Object.is;
    }

    var currentSlice = selector(state);

    function listenerToAdd() {
      var nextSlice = selector(state);

      if (!equalityFn(currentSlice, nextSlice)) {
        var _previousSlice = currentSlice;
        listener(currentSlice = nextSlice, _previousSlice);
      }
    }

    listeners.add(listenerToAdd);
    return function () {
      return listeners.delete(listenerToAdd);
    };
  };

  var subscribe = function subscribe(listener, selector, equalityFn) {
    if (selector || equalityFn) {
      return subscribeWithSelector(listener, selector, equalityFn);
    }

    listeners.add(listener);
    return function () {
      return listeners.delete(listener);
    };
  };

  var destroy = function destroy() {
    return listeners.clear();
  };

  var api = {
    setState: setState,
    getState: getState,
    subscribe: subscribe,
    destroy: destroy
  };
  state = createState(setState, getState, api);
  return api;
}

function create(createState) {
  var api = typeof createState === 'function' ? create$1(createState) : createState;
  var source = react.unstable_createMutableSource(api, function () {
    return api.getState();
  });
  var FUNCTION_SYMBOL = Symbol();
  var functionMap = new WeakMap();

  var useStore = function useStore(selector, equalityFn) {
    if (selector === void 0) {
      selector = api.getState;
    }

    if (equalityFn === void 0) {
      equalityFn = Object.is;
    }

    var getSnapshot = react.useMemo(function () {
      var lastSlice;
      return function () {
        var slice = lastSlice;

        try {
          slice = selector(api.getState());

          if (lastSlice !== undefined && equalityFn(lastSlice, slice)) {
            slice = lastSlice;
          } else {
            lastSlice = slice;
          }
        } catch (error) {}

        if (typeof slice === 'function') {
          var _wrappedFunction;

          if (functionMap.has(slice)) {
            return functionMap.get(slice);
          }

          var wrappedFunction = (_wrappedFunction = {}, _wrappedFunction[FUNCTION_SYMBOL] = slice, _wrappedFunction);
          functionMap.set(slice, wrappedFunction);
          return wrappedFunction;
        }

        return slice;
      };
    }, [selector, equalityFn]);
    var snapshot = react.unstable_useMutableSource(source, getSnapshot, api.subscribe);

    if (snapshot && snapshot[FUNCTION_SYMBOL]) {
      return snapshot[FUNCTION_SYMBOL];
    }

    return snapshot;
  };

  Object.assign(useStore, api);
  return useStore;
}

exports.default = create;
